==> fred_frgal-3.15.0.tar.gz: fred_frgal-3.15.0/frgal/tests/test_utils.py <==
import asyncio
from functools import partial
from typing import cast
from unittest import IsolatedAsyncioTestCase, TestCase
from unittest.mock import AsyncMock as UnittestAsyncMock, Mock, call, sentinel

from asyncstdlib import iter as aiter
from testfixtures import ShouldNotWarn, ShouldWarn

from frgal.aio import AsyncGrpcClient
from frgal.client import GrpcClient
from frgal.decoder import GrpcDecoder
from frgal.utils import (AsyncMock, AsyncTestClientMixin, CallableMock, MockingChannel, TestClientMixin, get_call_id,
                         get_mock_awaits, make_awaitable)

from .proto.example_pb2 import Reply, SimpleMessage  # type: ignore[attr-defined]


class GetCallIdTest(TestCase):
    def test_get_call_id(self):
        call_id = get_call_id()

        self.assertIsInstance(call_id, str)
        self.assertEqual(len(call_id), 4)

    def test_random(self):
        self.assertNotEqual(get_call_id(), get_call_id())


class CallableMockTest(IsolatedAsyncioTestCase):
    def test_call(self):
        a_callable = Mock(return_value=sentinel.reply)
        a_call = CallableMock('/Coffee/make', a_callable, exhaust_iterables=False)

        self.assertEqual(a_call(sentinel.request), sentinel.reply)

        self.assertEqual(a_callable.mock_calls, [call(sentinel.request)])

    async def test_exhaust(self):
        a_mock = Mock()
        a_call = CallableMock('/Coffee/make', a_mock, exhaust_iterables=True)
        data = (
            (123, 123),
            ("string", "string"),
            ((1, 2, 3), (1, 2, 3)),
            ([1, 2, 3], [1, 2, 3]),
            (iter([1, 2, 3]), (1, 2, 3)),
            (aiter([1, 2, 3]), (1, 2, 3)),
        )
        for ain, aout in data:
            with self.subTest(ain=ain, aout=aout):
                a_mock.return_value = make_awaitable(None)
                await a_call(ain)
                self.assertEqual(a_mock.mock_calls[-1], call(aout))

    async def test_not_exhaust(self):
        a_mock = Mock()
        a_call = CallableMock('/Coffee/make', a_mock, exhaust_iterables=False)
        data = ((1, 2, 3), iter([1, 2, 3]), aiter([1, 2, 3]))
        for ain in data:
            with self.subTest(ain=ain):
                a_mock.return_value = make_awaitable(None)
                await a_call(ain)
                self.assertIs(a_mock.mock_calls[-1][1][0], ain)

    async def test_exhaust_not_set_static(self):
        a_mock = Mock()
        a_call = CallableMock('/Coffee/make', a_mock)
        data = (sentinel.request, (1, 2, 3))
        for ain in data:
            with self.subTest(ain=ain):
                a_mock.return_value = make_awaitable(None)
                with ShouldNotWarn():
                    await a_call(ain)
                self.assertIs(a_mock.mock_calls[-1][1][0], ain)

    async def test_exhaust_not_set_iterable(self):
        a_mock = Mock()
        a_call = CallableMock('/Coffee/make', a_mock)
        data = (iter([1, 2, 3]), aiter([1, 2, 3]))
        for ain in data:
            with self.subTest(ain=ain):
                a_mock.return_value = make_awaitable(None)
                with ShouldWarn(DeprecationWarning(
                    'Default value of `exhaust_iterables` will change to True in next major release.'
                    'If you want to maintain the current behaviour, set it to False explicitely.')
                ):
                    await a_call(ain)
                self.assertIs(a_mock.mock_calls[-1][1][0], ain)

    async def test_exhaust_exception(self):
        async def exception_requests():
            raise RuntimeError
            yield None  # type: ignore # pragma: no cover

        a_mock = Mock()
        a_call = CallableMock('/Coffee/make', a_mock, exhaust_iterables=True)

        with self.assertRaises(RuntimeError):
            await a_call(exception_requests())
        a_mock.assert_not_called()


class MockingChannelTest(TestCase):
    def _test_callable(self, name: str) -> None:
        client = Mock(name='client')
        channel = MockingChannel(client)

        a_callable = getattr(channel, name)('/Coffee/make')

        a_call = cast(partial, a_callable.call)
        self.assertEqual(a_call.func, client.mock)
        self.assertEqual(a_call.args, ())
        self.assertEqual(a_call.keywords, {'method': '/Coffee/make'})

    def test_unary_unary(self):
        self._test_callable(name='unary_unary')

    def test_unary_stream(self):
        self._test_callable(name='unary_stream')

    def test_stream_unary(self):
        self._test_callable(name='stream_unary')

    def test_stream_stream(self):
        self._test_callable(name='stream_stream')


class CoffeeStub:
    """A simple stub for testing."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.make = channel.unary_unary('/Coffee/make')


class TestClient(TestClientMixin, GrpcClient):
    """Simple test client to test a TestClientMixin."""

    exhaust_iterables = False


class TestClientMixinTest(TestCase):
    def test_connect(self):
        client = TestClient(sentinel.netloc, sentinel.decoder, stub_cls=CoffeeStub)

        client.connect()

        self.assertIsInstance(client.channel, MockingChannel)

    def test_call(self):
        client = TestClient(sentinel.netloc, GrpcDecoder(), stub_cls=CoffeeStub)
        request = SimpleMessage(value='10')
        reply = Reply()
        reply.data.result = 'Espresso!'
        client.mock.return_value = reply

        result = client.call('make', request)

        self.assertEqual(result, 'Espresso!')
        self.assertEqual(client.mock.mock_calls, [call(request, method='/Coffee/make', timeout=None)])

    def test_call_future(self):
        client = TestClient(sentinel.netloc, GrpcDecoder(), stub_cls=CoffeeStub)
        request = SimpleMessage(value='10')
        reply = Reply()
        reply.data.result = 'Espresso!'
        client.mock.return_value = reply

        result = client.call_future('make', request)

        self.assertEqual(result.result(), 'Espresso!')
        self.assertEqual(client.mock.mock_calls, [call(request, method='/Coffee/make', timeout=None)])

    def test_call_stream(self):
        client = TestClient(sentinel.netloc, GrpcDecoder(), stub_cls=CoffeeStub)
        request = SimpleMessage(value='10')
        reply = Reply()
        reply.data.result = 'Espresso!'
        client.mock.return_value = [reply, reply]

        result = client.call_stream('make', request)

        self.assertEqual(tuple(result), ('Espresso!', 'Espresso!'))
        self.assertEqual(client.mock.mock_calls, [call(request, method='/Coffee/make', timeout=None)])


class AsyncTestClient(AsyncTestClientMixin, AsyncGrpcClient):
    """Simple test client to test a AsyncTestClientMixin."""

    exhaust_iterables = False


class AsyncTestClientMixinTest(IsolatedAsyncioTestCase):
    def test_init(self):
        client = AsyncTestClient(sentinel.netloc, GrpcDecoder(), stub_cls=CoffeeStub)
        self.assertIsInstance(client.mock, AsyncMock)
        self.assertIsInstance(client.channel, MockingChannel)
        self.assertIsInstance(client.channel._loop, asyncio.BaseEventLoop)


class AsyncMockTest(IsolatedAsyncioTestCase):
    def setUp(self):
        self.mock = AsyncMock(spec=('call',))
        self.mock.call.exhaust_iterables = False

    async def test_awaitable(self):
        self.mock.call.return_value = make_awaitable(sentinel.result)
        self.assertEqual(await self.mock.call(), sentinel.result)

    async def test_async_iterable(self):
        async def async_gen():
            yield sentinel.result
        self.mock.call.return_value = async_gen()
        self.assertEqual([i async for i in self.mock.call()], [sentinel.result])

    async def test_deprecation(self):
        self.mock.call.return_value = sentinel.result
        with ShouldWarn(DeprecationWarning(
            'Automagical transformation to awaitables is deprecated '
            'and will be removed in the next major release.')
        ):
            self.assertEqual(await self.mock.call(), sentinel.result)


class GetMockAwaitsTest(IsolatedAsyncioTestCase):
    async def test_await(self):
        mock = UnittestAsyncMock()
        await mock(sentinel.arg)

        self.assertEqual(get_mock_awaits(mock), [call(sentinel.arg)])

    async def test_await_method(self):
        mock = UnittestAsyncMock()
        await mock.foo(sentinel.arg)

        self.assertEqual(get_mock_awaits(mock), [call.foo(sentinel.arg)])

    async def test_await_children(self):
        mock = UnittestAsyncMock()
        await mock.sub.foo(sentinel.arg)

        self.assertEqual(get_mock_awaits(mock), [call.sub.foo(sentinel.arg)])

    async def test_nested_awaits(self):
        mock = UnittestAsyncMock()
        result = await mock.sub(sentinel.arg)
        await result.foo(sentinel.other)

        self.assertEqual(get_mock_awaits(mock), [call.sub(sentinel.arg), call.sub().foo(sentinel.other)])

    async def test_no_await(self):
        """Test coroutine creation itself is not listed."""
        mock = UnittestAsyncMock()
        coro = mock.foo()

        self.assertEqual(get_mock_awaits(mock), [])
        # Await coroutine to prevent RuntimeWarning
        await coro

    async def test_multiple_awaits(self):
        mock = UnittestAsyncMock()
        await mock(sentinel.arg)
        await mock(sentinel.arg)
        await mock(sentinel.another)

        self.assertEqual(get_mock_awaits(mock), [call(sentinel.arg), call(sentinel.arg), call(sentinel.another)])

==> fred_frgal-3.15.1.tar.gz: fred_frgal-3.15.1/frgal/tests/test_utils.py <==
import asyncio
from functools import partial
from typing import cast
from unittest import IsolatedAsyncioTestCase, TestCase
from unittest.mock import AsyncMock as UnittestAsyncMock, Mock, call, sentinel

from asyncstdlib import iter as aiter
from testfixtures import ShouldNotWarn, ShouldWarn

from frgal.aio import AsyncGrpcClient
from frgal.client import GrpcClient
from frgal.decoder import GrpcDecoder
from frgal.utils import (AsyncMock, AsyncTestClientMixin, CallableMock, MockingChannel, TestClientMixin,
                         get_mock_awaits, make_awaitable)

from .proto.example_pb2 import Reply, SimpleMessage  # type: ignore[attr-defined]


class CallableMockTest(IsolatedAsyncioTestCase):
    def test_call(self):
        a_callable = Mock(return_value=sentinel.reply)
        a_call = CallableMock('/Coffee/make', a_callable, exhaust_iterables=False)

        self.assertEqual(a_call(sentinel.request), sentinel.reply)

        self.assertEqual(a_callable.mock_calls, [call(sentinel.request)])

    async def test_exhaust(self):
        a_mock = Mock()
        a_call = CallableMock('/Coffee/make', a_mock, exhaust_iterables=True)
        data = (
            (123, 123),
            ("string", "string"),
            ((1, 2, 3), (1, 2, 3)),
            ([1, 2, 3], [1, 2, 3]),
            (iter([1, 2, 3]), (1, 2, 3)),
            (aiter([1, 2, 3]), (1, 2, 3)),
        )
        for ain, aout in data:
            with self.subTest(ain=ain, aout=aout):
                a_mock.return_value = make_awaitable(None)
                await a_call(ain)
                self.assertEqual(a_mock.mock_calls[-1], call(aout))

    async def test_not_exhaust(self):
        a_mock = Mock()
        a_call = CallableMock('/Coffee/make', a_mock, exhaust_iterables=False)
        data = ((1, 2, 3), iter([1, 2, 3]), aiter([1, 2, 3]))
        for ain in data:
            with self.subTest(ain=ain):
                a_mock.return_value = make_awaitable(None)
                await a_call(ain)
                self.assertIs(a_mock.mock_calls[-1][1][0], ain)

    async def test_exhaust_not_set_static(self):
        a_mock = Mock()
        a_call = CallableMock('/Coffee/make', a_mock)
        data = (sentinel.request, (1, 2, 3))
        for ain in data:
            with self.subTest(ain=ain):
                a_mock.return_value = make_awaitable(None)
                with ShouldNotWarn():
                    await a_call(ain)
                self.assertIs(a_mock.mock_calls[-1][1][0], ain)

    async def test_exhaust_not_set_iterable(self):
        a_mock = Mock()
        a_call = CallableMock('/Coffee/make', a_mock)
        data = (iter([1, 2, 3]), aiter([1, 2, 3]))
        for ain in data:
            with self.subTest(ain=ain):
                a_mock.return_value = make_awaitable(None)
                with ShouldWarn(DeprecationWarning(
                    'Default value of `exhaust_iterables` will change to True in next major release.'
                    'If you want to maintain the current behaviour, set it to False explicitely.')
                ):
                    await a_call(ain)
                self.assertIs(a_mock.mock_calls[-1][1][0], ain)

    async def test_exhaust_exception(self):
        async def exception_requests():
            raise RuntimeError
            yield None  # type: ignore # pragma: no cover

        a_mock = Mock()
        a_call = CallableMock('/Coffee/make', a_mock, exhaust_iterables=True)

        with self.assertRaises(RuntimeError):
            await a_call(exception_requests())
        a_mock.assert_not_called()


class MockingChannelTest(TestCase):
    def _test_callable(self, name: str) -> None:
        client = Mock(name='client')
        channel = MockingChannel(client)

        a_callable = getattr(channel, name)('/Coffee/make')

        a_call = cast(partial, a_callable.call)
        self.assertEqual(a_call.func, client.mock)
        self.assertEqual(a_call.args, ())
        self.assertEqual(a_call.keywords, {'method': '/Coffee/make'})

    def test_unary_unary(self):
        self._test_callable(name='unary_unary')

    def test_unary_stream(self):
        self._test_callable(name='unary_stream')

    def test_stream_unary(self):
        self._test_callable(name='stream_unary')

    def test_stream_stream(self):
        self._test_callable(name='stream_stream')


class CoffeeStub:
    """A simple stub for testing."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.make = channel.unary_unary('/Coffee/make')


class TestClient(TestClientMixin, GrpcClient):
    """Simple test client to test a TestClientMixin."""

    exhaust_iterables = False


class TestClientMixinTest(TestCase):
    def test_connect(self):
        client = TestClient(sentinel.netloc, sentinel.decoder, stub_cls=CoffeeStub)

        client.connect()

        self.assertIsInstance(client.channel, MockingChannel)

    def test_call(self):
        client = TestClient(sentinel.netloc, GrpcDecoder(), stub_cls=CoffeeStub)
        request = SimpleMessage(value='10')
        reply = Reply()
        reply.data.result = 'Espresso!'
        client.mock.return_value = reply

        result = client.call('make', request)

        self.assertEqual(result, 'Espresso!')
        self.assertEqual(client.mock.mock_calls, [call(request, method='/Coffee/make', timeout=None)])

    def test_call_future(self):
        client = TestClient(sentinel.netloc, GrpcDecoder(), stub_cls=CoffeeStub)
        request = SimpleMessage(value='10')
        reply = Reply()
        reply.data.result = 'Espresso!'
        client.mock.return_value = reply

        result = client.call_future('make', request)

        self.assertEqual(result.result(), 'Espresso!')
        self.assertEqual(client.mock.mock_calls, [call(request, method='/Coffee/make', timeout=None)])

    def test_call_stream(self):
        client = TestClient(sentinel.netloc, GrpcDecoder(), stub_cls=CoffeeStub)
        request = SimpleMessage(value='10')
        reply = Reply()
        reply.data.result = 'Espresso!'
        client.mock.return_value = [reply, reply]

        result = client.call_stream('make', request)

        self.assertEqual(tuple(result), ('Espresso!', 'Espresso!'))
        self.assertEqual(client.mock.mock_calls, [call(request, method='/Coffee/make', timeout=None)])


class AsyncTestClient(AsyncTestClientMixin, AsyncGrpcClient):
    """Simple test client to test a AsyncTestClientMixin."""

    exhaust_iterables = False


class AsyncTestClientMixinTest(IsolatedAsyncioTestCase):
    def test_init(self):
        client = AsyncTestClient(sentinel.netloc, GrpcDecoder(), stub_cls=CoffeeStub)
        self.assertIsInstance(client.mock, AsyncMock)
        self.assertIsInstance(client.channel, MockingChannel)
        self.assertIsInstance(client.channel._loop, asyncio.BaseEventLoop)


class AsyncMockTest(IsolatedAsyncioTestCase):
    def setUp(self):
        self.mock = AsyncMock(spec=('call',))
        self.mock.call.exhaust_iterables = False

    async def test_awaitable(self):
        self.mock.call.return_value = make_awaitable(sentinel.result)
        self.assertEqual(await self.mock.call(), sentinel.result)

    async def test_async_iterable(self):
        async def async_gen():
            yield sentinel.result
        self.mock.call.return_value = async_gen()
        self.assertEqual([i async for i in self.mock.call()], [sentinel.result])

    async def test_deprecation(self):
        self.mock.call.return_value = sentinel.result
        with ShouldWarn(DeprecationWarning(
            'Automagical transformation to awaitables is deprecated '
            'and will be removed in the next major release.')
        ):
            self.assertEqual(await self.mock.call(), sentinel.result)


class GetMockAwaitsTest(IsolatedAsyncioTestCase):
    async def test_await(self):
        mock = UnittestAsyncMock()
        await mock(sentinel.arg)

        self.assertEqual(get_mock_awaits(mock), [call(sentinel.arg)])

    async def test_await_method(self):
        mock = UnittestAsyncMock()
        await mock.foo(sentinel.arg)

        self.assertEqual(get_mock_awaits(mock), [call.foo(sentinel.arg)])

    async def test_await_children(self):
        mock = UnittestAsyncMock()
        await mock.sub.foo(sentinel.arg)

        self.assertEqual(get_mock_awaits(mock), [call.sub.foo(sentinel.arg)])

    async def test_nested_awaits(self):
        mock = UnittestAsyncMock()
        result = await mock.sub(sentinel.arg)
        await result.foo(sentinel.other)

        self.assertEqual(get_mock_awaits(mock), [call.sub(sentinel.arg), call.sub().foo(sentinel.other)])

    async def test_no_await(self):
        """Test coroutine creation itself is not listed."""
        mock = UnittestAsyncMock()
        coro = mock.foo()

        self.assertEqual(get_mock_awaits(mock), [])
        # Await coroutine to prevent RuntimeWarning
        await coro

    async def test_multiple_awaits(self):
        mock = UnittestAsyncMock()
        await mock(sentinel.arg)
        await mock(sentinel.arg)
        await mock(sentinel.another)

        self.assertEqual(get_mock_awaits(mock), [call(sentinel.arg), call(sentinel.arg), call(sentinel.another)])
